/*
 * BASED ON PSPDEV TEXT SAMPLE
 * Example begins blitting strings to XMB on LTRIGGER press
 * by John Lazlow (aka OldPrisoneR)
*/

#include <pspkernel.h>
#include <pspdisplay.h>
#include <string.h>
#include <pspctrl.h>
#include <pspgu.h>
#include "font.c"

PSP_MODULE_INFO("MultiDisplayList Sample", 0, 0, 1);
PSP_NO_CREATE_MAIN_THREAD();

//That will be enough for our purposes
static unsigned int __attribute__((aligned(16))) list[2048]; 

/**/

char running = 1;
int DISP_thid = -1;
char en = 0;

PspGeContext __attribute__((aligned(16))) context;

static int fontwidthtab[128] = {
	10, 10, 10, 10, 
	10, 10, 10, 10,
	10, 10, 10, 10, 
	10, 10, 10, 10,

	10, 10, 10, 10, 
	10, 10, 10, 10,
	10, 10, 10, 10,
	10, 10, 10, 10,

	10,  6,  8, 10, //   ! " #
	10, 10, 10,  6, // $ % & '
	10, 10, 10, 10, // ( ) * +
	 6, 10,  6, 10, // , - . /

	10, 10, 10, 10, // 0 1 2 3
	10, 10, 10, 10, // 6 5 8 7
	10, 10,  6,  6, // 10 9 : ;
	10, 10, 10, 10, // < = > ?

	16, 10, 10, 10, // @ A B C
	10, 10, 10, 10, // D E F G
	10,  6,  8, 10, // H I J K
	 8, 10, 10, 10, // L M N O

	10, 10, 10, 10, // P Q R S
	10, 10, 10, 12, // T U V W
	10, 10, 10, 10, // X Y Z [
	10, 10,  8, 10, // \ ] ^ _

	 6,  8,  8,  8, // ` a b c
	 8,  8,  6,  8, // d e f g
	 8,  6,  6,  8, // h i j k
	 6, 10,  8,  8, // l m n o

	 8,  8,  8,  8, // p q r s
	 8,  8,  8, 12, // t u v w
	 8,  8,  8, 10, // x y z {
	 8, 10,  8, 12  // | } ~  
};

void drawString(const char* text, int x, int y, unsigned int color, int fw) {
	int len = (int)strlen(text);
	if(!len) {
		return;
	}

	typedef struct {
		float s, t;
		unsigned int c;
		float x, y, z;
	} VERT;

	VERT* v = sceGuGetMemory(sizeof(VERT) * 2 * len);

	int i;
	for(i = 0; i < len; i++) {
		unsigned char c = (unsigned char)text[i];
		if(c < 32) {
			c = 0;
		} else if(c >= 128) {
			c = 0;
		}

		int tx = (c & 0x0F) << 4;
		int ty = (c & 0xF0);

		VERT* v0 = &v[i*2+0];
		VERT* v1 = &v[i*2+1];
		
		v0->s = (float)(tx + (fw ? ((16 - fw) >> 1) : ((16 - fontwidthtab[c]) >> 1)));
		v0->t = (float)(ty);
		v0->c = color;
		v0->x = (float)(x);
		v0->y = (float)(y);
		v0->z = 0.0f;

		v1->s = (float)(tx + 16 - (fw ? ((16 - fw) >> 1) : ((16 - fontwidthtab[c]) >> 1)));
		v1->t = (float)(ty + 16);
		v1->c = color;
		v1->x = (float)(x + (fw ? fw : fontwidthtab[c]));
		v1->y = (float)(y + 16);
		v1->z = 0.0f;

		x += (fw ? fw : fontwidthtab[c]);
	}

	sceGuDrawArray(GU_SPRITES, 
		GU_TEXTURE_32BITF | GU_COLOR_8888 | GU_VERTEX_32BITF | GU_TRANSFORM_2D,
		len * 2, 0, v
	);
}

int threadDisp(SceSize args, void *argp)
{
	while(en)
	{
	sceDisplayWaitVblankStart();
	
	sceGeSaveContext(&context);
	
	sceGuStart(GU_DIRECT, list);
	sceGuDisable(GU_DEPTH_TEST);
	sceGuEnable(GU_BLEND);
	sceGuBlendFunc(GU_ADD, GU_SRC_ALPHA, GU_ONE_MINUS_SRC_ALPHA, 0, 0);
	sceGuEnable(GU_TEXTURE_2D);
	sceGuTexMode(GU_PSM_8888, 0, 0, 0);
	sceGuTexImage(0, 256, 128, 256, font);
	sceGuTexFunc(GU_TFX_MODULATE, GU_TCC_RGBA);
	sceGuTexWrap(GU_REPEAT, GU_REPEAT);
	sceGuTexFilter(GU_NEAREST, GU_NEAREST);
	
		drawString("Opacity 100%", 0, 0, 0xFFFFFFFF, 0);
		drawString("Opacity  50%", 0, 8, 0x7FFFFFFF, 0);
		drawString("Opacity  10%", 0, 16, 0x18FFFFFF, 0);
	
	sceGuFinish();
	sceGuSync(0,0);
	
	sceGeRestoreContext(&context);
	
	};

return 0;
}

int threadMain(SceSize args, void *argp)
{
 
	sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);
	
	while(running)
	{
		SceCtrlData pad;
 
		sceCtrlPeekBufferPositive(&pad, 1);
		
		if(pad.Buttons & PSP_CTRL_LTRIGGER)
		{
			if (en)
			{
				en = 0;
				sceKernelWaitThreadEnd(DISP_thid, 0);
			} else {
				DISP_thid = sceKernelCreateThread("MDL Test DISPLAY", threadDisp, 33, 0x800, 0, NULL);
				if (DISP_thid >= 0) 
				{
				en = 1;
				sceKernelStartThread(DISP_thid, args, argp);
				}
			}
			
			while (pad.Buttons & PSP_CTRL_LTRIGGER) sceCtrlPeekBufferPositive(&pad, 1);
		
		}
		sceDisplayWaitVblankStart();
	}

	return 0;
}


int module_start(SceSize args, void *argp)
{
	int main_thid = sceKernelCreateThread("MDL Test", threadMain, 32, 0x800, 0, NULL);
	if (main_thid >= 0) sceKernelStartThread(main_thid, args, argp);
	
	return 0;
}

int module_stop(SceSize args, void *argp)
{
	running = 0;
	return 0;
}

